<%@ Page Language="C#" enableViewState="false" Debug="true" %>
<%@ Import Namespace="System"%>
<%@ Import Namespace="System.Xml"%>
<%@ Import Namespace="System.Text"%>
<%@ Import Namespace="System.Net"%>
<%@ Import Namespace="System.Web.Services.Protocols"%>
<%@ Import Namespace="Microsoft.Crm.Platform.Proxy"%>
<HTML>
	<HEAD>
		<title>Copying Address Information...</title>
		<link rel="stylesheet" type="text/css" href="/_common/styles/select.css">
		<link rel="stylesheet" type="text/css" href="/_nav/tabs.css">
		<link rel="stylesheet" type="text/css" href="/_nav/nav.css">
		<link rel="stylesheet" type="text/css" href="/_nav/menu.css">
		<link rel="stylesheet" type="text/css" href="/_common/styles/global.css">
		<link rel="stylesheet" type="text/css" href="/_forms/controls/form.css">
		
		<script language="C#" runat="server">
		// CRM WebService
		CrmService _service;
			
		void Page_Init() {
			string objectId =  Request.QueryString["oId"].ToString();
			string objectType = Request.QueryString["oType"].ToString();
			string serverName = string.Empty;
			string webServicesUrl = string.Empty;			
			string objectXml = string.Empty;
			string nameString = string.Empty;
			string companyName = string.Empty;
			
			// Buld the URL to the CRM web services:
			//	1) Read a key called 'CRMServer' from web.config
			//	2) The value of this key must start with either http:// or https://
			//	3) This value must contain the name of the CRM server (or host header)
			//	4) The IP port number can be added at the end of the value if necessary
			// Here is an example: <add key="CRMServer" value="http://crm:5555"/>
			if (ConfigurationSettings.AppSettings["CRMServer"] == null) {
				throw new ArgumentNullException("CRMServer", "The CRMServer key is missing from the web.config file or is empty.");
			}
			serverName = ConfigurationSettings.AppSettings["CRMServer"].ToString();
			if (!serverName.StartsWith("http://") && !serverName.StartsWith("https://")) {
				throw new ArgumentException("CRMServer", "The CRMServer key must start with http:// or https://");
			}
			if (!serverName.EndsWith("/")) {
				serverName += "/";
			}
			webServicesUrl = string.Concat(serverName, "/MSCRMServices/2006/CrmService.asmx" );

			// create the CrmService webservice
			_service = new CrmService( webServicesUrl );
			_service.Credentials = CredentialCache.DefaultCredentials;
			
			objectXml = RetrieveObjectXml( int.Parse( objectType ), objectId );
			nameString = RetrieveNameXml( int.Parse(objectType), objectId, objectXml );
			companyName = RetrieveCompanyName ( int.Parse( objectType ), objectId, objectXml );

			if ( objectXml.Length > 0 ) {
				CopyToClipboard( objectXml, string.Concat( nameString, companyName ) );
			}
		}
		
		
		/// <summary>
		/// Retrieves information about the entity using fetchXml.
		/// </summary>
		/// <param name="objectType">Entity type to retrieve information for.</param>
		/// <param name="objectId">GUID of the entity to retrieve information for.</param>
		/// <returns>ResultSet xml data for the entity.</returns>
		private string RetrieveObjectXml ( int objectType, string objectId ) {
			StringBuilder fetchXml = new StringBuilder();

			fetchXml.Append( "<fetch mapping='logical'><entity name=" );
			switch(objectType) {         
				case 1:
					fetchXml.Append( @"'account'><all-attributes/><filter type='and'><condition attribute = 'accountid' operator='eq' " );
					break;
				case 2:
					fetchXml.Append( @"'contact'><all-attributes/><filter type='and'><condition attribute = 'contactid' operator='eq' " );
					break;
				case 4:
					fetchXml.Append( @"'lead'><all-attributes/><filter type='and'><condition attribute = 'leadid' operator='eq' " );
					break;
			}
			
			fetchXml.AppendFormat( @"value='{0}'/></filter></entity></fetch>", objectId );
			
			// create the FetchXml request and execute.
			ExecuteFetchRequest request = new ExecuteFetchRequest();
			request.FetchXml = fetchXml.ToString();
			ExecuteFetchResponse response = ( ExecuteFetchResponse )_service.Execute( request );
			
			return response.FetchXmlResult;
		}
	
		/// <summary>
		/// This function will return the string containing only name information.
		/// Required because we need the name information of the Primary Contact of Account
		/// and the name information of Contact and Lead directly.
		/// </summary>
		/// <param name="objectType">Entity type to retrieve information for</param>
		/// <param name="objectId">GUID of the entity to retrieve information for</param>
		/// <param name="objectXml">ResultSet Xml.</param>
		/// <returns>Contact name</returns>
		private string RetrieveNameXml ( int objectType, string objectId, string objectXml ) {
			
			XmlDocument xmlDocument = new XmlDocument();

			// Account
			if ( objectType == 1 )  {
				xmlDocument.LoadXml ( objectXml );
				
				XmlNode contactNode = xmlDocument.SelectSingleNode( "//resultset/result/primarycontactid" );
				if ( contactNode != null ) {
					string primaryContact = contactNode.InnerText;
					objectXml = RetrieveObjectXml( 2, primaryContact );
				}
				else
				{
					return string.Empty;
				}
			}

			xmlDocument.LoadXml ( objectXml );
			
			StringBuilder addressText = new StringBuilder();

			// Salutation
			XmlNode currentXmlNode = xmlDocument.SelectSingleNode ( "//resultset/result/salutation" );
			if ( currentXmlNode != null ) {
				addressText.Append ( string.Concat( currentXmlNode.InnerText, " " ) );
			}
			// First Name
			currentXmlNode = xmlDocument.SelectSingleNode ( "//resultset/result/firstname" );
			if ( currentXmlNode != null ) {
				addressText.Append ( string.Concat( currentXmlNode.InnerText, " " ) );
			}
			// Middle Name
			currentXmlNode = xmlDocument.SelectSingleNode ( "//resultset/result/middlename" );
			if ( currentXmlNode != null ) {
				addressText.Append ( string.Concat( currentXmlNode.InnerText, " " ) );
			}
			// Last Name
			currentXmlNode = xmlDocument.SelectSingleNode ( "//resultset/result/lastname" );
			if ( currentXmlNode != null ) {
				addressText.Append ( currentXmlNode.InnerText );
			}
			
			// add line break
			if (addressText.Length > 0 ) {
				addressText.Append ( "\\r\\n" );
			}
				
			return addressText.ToString();
		}

		/// <summary>
		/// Get the company name
		/// </summary>
		/// <param name="objectType">Entity type to retrieve information for</param>
		/// <param name="objectId">GUID of the entity to retrieve information for</param>
		/// <param name="objectXml">ResultSet Xml</param>
		/// <returns>Company name</returns>
		private string RetrieveCompanyName( int objectType, string objectId, string objectXml ) {
			
			string companyName = string.Empty;
			XmlDocument xmlDocument = new XmlDocument();
			XmlNode currentNode = null;
			xmlDocument.LoadXml ( objectXml );

			// account
			if (objectType == 1 )  {				
				currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/name" );
			}
			// contact
			else if (objectType == 2) {				
				currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/parentcustomerid" );
				
				// Retrieve the accountid and then the account name
				if ( currentNode != null )  {
					xmlDocument.LoadXml( RetrieveObjectXml( 1, currentNode.InnerText ) );

					currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/name" );
				}
			}
			//lead
			else if ( objectType == 4 )  {
				currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/companyname" );
			}
			
			if ( currentNode != null ) {
				companyName = string.Concat( currentNode.InnerText, "\\r\\n" );
			}

			return companyName;
		}
		
		/// <summary>
		/// Get the address from the ResultSet and save to clipboard.
		/// </summary>
		/// <param name="objectXml">ResultSet Xml</param>
		/// <param name="nameString">Contact name</param>
		private void CopyToClipboard ( string objectXml, string nameString )  {

			// get the address information from the xml
			StringBuilder addressText = new StringBuilder( nameString );
			XmlDocument xmlDocument = new XmlDocument ();
			xmlDocument.LoadXml ( objectXml );
			
			// address line 1
			XmlNode currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/address1_line1" );
			if ( currentNode != null ) {
				addressText.Append ( currentNode.InnerText );
			}
			
			// address  line2
			currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/address1_line2" );
			if ( currentNode != null ) {
				addressText.Append ( string.Concat( ", ", currentNode.InnerText ) );
			}

			// address line3
			currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/address1_line3" );
			if ( currentNode != null ) {
				addressText.Append ( string.Concat( ", ", currentNode.InnerText ) );
			}
			addressText.Append ( "\\r\\n" );
			
			// city
			currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/address1_city" );
			if ( currentNode != null ) {
				addressText.Append ( string.Concat( currentNode.InnerText, "," ) );
			}

			// state
			currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/address1_stateorprovince" );
			if ( currentNode != null ) {
				addressText.Append ( string.Concat( " ", currentNode.InnerText, " " ) );
			}

			// postal code
			currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/address1_postalcode" );
			if ( currentNode != null ) {
				addressText.Append ( string.Concat( " ", currentNode.InnerText ) );
			}
			addressText.Append ( "\\r\\n" );
	
			// country
			currentNode = xmlDocument.SelectSingleNode ( "/resultset/result/address1_country" );
			if ( currentNode != null ) {
				addressText.Append ( currentNode.InnerText );
			}

			// pass the info to the clipboard
			Response.Write ( "<script>window.clipboardData.setData( 'Text', \"" + addressText.ToString() + "\");<" + "/script>" );
		}
		
		[System.Diagnostics.DebuggerStepThroughAttribute()]
		[System.ComponentModel.DesignerCategoryAttribute("code")]
		[System.Web.Services.WebServiceBindingAttribute(Name="CrmServiceSoap", Namespace="http://schemas.microsoft.com/crm/2006/WebServices")]
		[System.Xml.Serialization.XmlIncludeAttribute(typeof(CrmReference))]
		public class CrmService : System.Web.Services.Protocols.SoapHttpClientProtocol {
	        
			public CallerId CallerIdValue;
	        
			/// <remarks/>
			public CrmService(string webServiceUrl) {
				this.Url = webServiceUrl;
			}
	        
			/// <remarks/>
			[System.Web.Services.Protocols.SoapHeaderAttribute("CallerIdValue")]
			[System.Web.Services.Protocols.SoapDocumentMethodAttribute("http://schemas.microsoft.com/crm/2006/WebServices/Execute", Use=System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle=System.Web.Services.Protocols.SoapParameterStyle.Bare)]
			[return: System.Xml.Serialization.XmlElementAttribute("Response", Namespace="http://schemas.microsoft.com/crm/2006/WebServices")]
			public CRMResponse Execute([System.Xml.Serialization.XmlElementAttribute(Namespace="http://schemas.microsoft.com/crm/2006/WebServices")] CRMRequest Request) {
				object[] results = this.Invoke( "Execute", new object[] { Request } );
				return ( ( CRMResponse )( results[ 0 ] ) );
			}        
		}
	    
		[System.Xml.Serialization.XmlTypeAttribute(Namespace="http://schemas.microsoft.com/crm/2006/CoreTypes")]
		[System.Xml.Serialization.XmlRootAttribute(Namespace="http://schemas.microsoft.com/crm/2006/WebServices", IsNullable=true)]
		public class CallerId : System.Web.Services.Protocols.SoapHeader {
	        
			/// <remarks/>
			public System.Guid CallerGuid;
		}
	    

		[System.Xml.Serialization.XmlIncludeAttribute(typeof(ExecuteFetchResponse))]
		public abstract class CRMResponse { }

		[System.Xml.Serialization.XmlTypeAttribute(Namespace="http://schemas.microsoft.com/crm/2006/WebServices")]
		public class CrmReference {
	        
			/// <remarks/>
			[System.Xml.Serialization.XmlAttributeAttribute()]
			public bool IsNull;
	        
			/// <remarks/>
			[System.Xml.Serialization.XmlIgnoreAttribute()]
			public bool IsNullSpecified;
	        
			/// <remarks/>
			[System.Xml.Serialization.XmlAttributeAttribute()]
			public string name;
	        
			/// <remarks/>
			[System.Xml.Serialization.XmlAttributeAttribute()]
			public string type;
	        
			/// <remarks/>
			[System.Xml.Serialization.XmlAttributeAttribute()]
			public int dsc;
	        
			/// <remarks/>
			[System.Xml.Serialization.XmlIgnoreAttribute()]
			public bool dscSpecified;
	        
			/// <remarks/>
			[System.Xml.Serialization.XmlTextAttribute()]
			public System.Guid Value;
		}

		[System.Xml.Serialization.XmlTypeAttribute(Namespace="http://schemas.microsoft.com/crm/2006/WebServices")]
		public class ExecuteFetchResponse : CRMResponse {        
			/// <remarks/>
			public string FetchXmlResult;
		}

		[System.Xml.Serialization.XmlIncludeAttribute(typeof(ExecuteFetchRequest))]
		public abstract class CRMRequest {	}

		[System.Xml.Serialization.XmlTypeAttribute(Namespace="http://schemas.microsoft.com/crm/2006/WebServices")]
		public class ExecuteFetchRequest : CRMRequest {        
			/// <remarks/>
			public string FetchXml;
		}		
		</script>
	</HEAD>
	<body>
		<table cellspacing="0" cellpadding="0" width="100%" height=100%>
			<tr>
				<td width="100%" valign=top>
					<table cellspacing="0" cellpadding="5" class="mnubar">
						<tr  align=left>
							<td align="left" style="font-family:tahoma;" class="mnuTitle mnuLeft">The Address information has been copied into the clipboard.</td>
						</tr>
					</table>
				</td>
			</tr>
			<tr height=20%><td>&nbsp;</td></tr>
			<tr>
				<td align="center"><button onclick="window.close()" title="Close the window">Close</button></td>
			</tr>
		</table>
	</body>
</HTML>
